# -*- coding: cp1252 -*-
# (C) by Fred Bayer <fred@bayerf.de>
#
# Support library for F4LW.
# You need the pyserial library available at
# pyserial.sourceforge.net

import sys
import serial
import time
import random

ACK=chr(6)
STX=chr(2)
ETX=chr(3)

BLOCKSIZE=64
BAUDRATE=19200
EEPROM_SIZE=2**16

def openSerial(**kwargs):
  return serial.Serial(0, baudrate=BAUDRATE, **kwargs)
  
def waitForAck(ser):
  ser.timeout = 10
  resp = ser.read(1)
  if resp == '':
    raise Exception, "Timeout!"
  elif resp != ACK:
    raise Exception, "ACK expected"

def uploadFile(filename):
  uploadMem(file(filename,"rb").read(), True)

def uploadMem(bytes, sendLen=False):
  size = len(bytes)
  if size > EEPROM_SIZE:
    raise "File too large"
  blocks = size // BLOCKSIZE
  if size % BLOCKSIZE:
    blocks += 1
    bytes += "\0" * (BLOCKSIZE-size%BLOCKSIZE)
  print "Will send %d blocks." % blocks
  ser = openSerial()
  try:
    if sendLen:
      ser.write(chr(blocks // 256) + chr(blocks % 256))
      waitForAck(ser)
    for i in range(blocks):
      ser.write(bytes[i*BLOCKSIZE:(i+1)*BLOCKSIZE])
      waitForAck(ser)
    print "Finished!"
  finally:
    ser.close() 

def uploadRTC(ts):
  ser = openSerial()
  try:
    ser.write(ts)
    waitForAck(ser)
    print "Finished!"
  finally:
    ser.close() 

def tickerFile(filename, *rest):
  tickerString(file(filename).read(), *rest)

def tickerString(s, delay=0.4):
  ser = openSerial()
  try:
    for c in s:
      if c=='\n':
        for i in range(0,4):
          ser.write(' ')
          time.sleep(delay)
      else:
        ser.write(c)
        time.sleep(delay)
  finally:
    ser.close()  

def displayString(s):  
  ser = openSerial()
  try:
    ser.write(s)
  finally:
    ser.close()  

def sync_server(handlerclass):
  ser = openSerial(timeout=1)
  resp = ' '
  try:
    while True:
      while resp != STX:
        resp = ser.read(1)
      handler = handlerclass()
      while True:
        ser.write(handler.display())
        resp = ''
        while resp == '' or resp == STX:
          resp = ser.read(1)
        if resp == ETX:
          break
        handler.input(resp)
      del handler
  finally:
    ser.close()

